unit Poly;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

type
  TPolygons = class(TGraphicControl)
  private
    rx,ry,__StartAngle,__AngleStep,locx,locy,_PolyCount,_StartAngleChange,
    _StartPolyCount: integer;
    Trans,_AutoPolyCount, _WidthHeightChange: boolean;
    _WidthChange, _HeightChange: integer;
    procedure set_Trans(b: boolean);
    procedure set_StartAngle(_StartAngle: integer);
    procedure set_AngleStep(_AngleStep: integer);
    procedure set_pen(p: TPen);
    function get_pen: TPen;
    procedure set_PolyCount(c: integer);
    procedure set_StartPolyCount(c: integer);
    procedure set_WidthChange(w: integer);
    procedure set_HeightChange(h: integer);
    procedure set_StartAngleChange(s: integer);
    procedure set_AutoPolyCount(b: boolean);
    procedure set_WidthHeightChange(b:boolean);
    procedure FixIt;
  protected

  public
    constructor Create(AOwner: TComponent);override;
    procedure draw;
    procedure paint;override;
  published
    property Color;
    property ParentColor;
    property Align;
    property Visible;
    property enabled;
    property ShowHint;
    property StartAngle: integer read __StartAngle write set_StartAngle;
    property AngleStep: integer read __AngleStep write set_AngleStep;
    Property Pen: TPen read get_pen write set_pen;
    property Transparent: boolean read Trans write set_Trans;
    property PolyCount: integer read _PolyCount write Set_PolyCount;
    property StartPolyCount: integer read _StartPolyCount write Set_StartPolyCount;
    property WidthChange: integer read _WidthChange write set_WidthChange;
    property HeightChange: integer read _HeightChange write set_HeightChange;
    property WidthHeightChange: boolean read _WidthHeightChange write set_WidthHeightChange;
    property StartAngleChange : integer read _StartAngleChange write set_StartAngleChange;
    property AutoPolyCount: boolean read _AutoPolyCount write set_AutoPolyCount;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TPolygons]);
end;

constructor TPolygons.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  __StartAngle:=0;
  __AngleStep:=90;
  Width:=200;
  Height:=200;
  Trans:=True;
  _StartAngleChange:=30;
  _PolyCount:=1;
  _StartPolyCount:=1;
  _AutoPolyCount:=true;
  _StartAngleChange:=5;
  _WidthChange:=7;
  _HeightChange:=7;
end;

procedure TPolygons.Draw;
var
  i,x,y,counter,stangle,fx,fy: integer;
begin
  Canvas.Brush.Color:=Color;
  if Trans=False then Canvas.FillRect(Rect(0,0,width,height));
  Locx:=width div 2;
  locy:=height div 2;
  rx:=width div 2;
  ry:=height div 2;
  counter:=0;
  stangle:=__Startangle;
  while ((counter<_PolyCount) and (AutoPolyCount=False)) or
   ((AutoPolyCount=True) and (rx>0) and (ry>0)) do
  begin
    i:=stangle;
    while (i<=360+Stangle) do
    begin
      x:=round(cos(i*pi/180)*rx+locx);
      y:=round(sin(i*pi/180)*ry+locy);
      if i=stangle then
      begin
        Canvas.MoveTo(x,y);
        fx:=x;
        fy:=y;
      end;
      if (_StartPolyCount-1<=counter) then Canvas.LineTo(x,y);
      i:=i+AngleStep;
    end;
    if (x<>fx) or (y<>fy) then Canvas.LineTo(fx,fy);
    inc(counter);
    dec(rx,_WidthChange);
    dec(ry,_HeightChange);
    inc(stangle,_StartAngleChange);
    if stangle>360 then dec(stangle,360);
  end;
end;

procedure TPolygons.Paint;
begin
  draw;
end;
procedure TPolygons.set_StartAngle(_StartAngle: integer);
begin
  if (_StartAngle>=0) and (_StartAngle<360) then __StartAngle:=_StartAngle;
  Invalidate;
end;

procedure TPolygons.set_AngleStep(_AngleStep: integer);
begin
  if (_AngleStep>0) and (_AngleStep<360) then __AngleStep:=_AngleStep;
  Invalidate;
end;

procedure TPolygons.set_pen(p: TPen);
begin
  Canvas.Pen:=p;
  Invalidate;
end;

function TPolygons.get_pen: TPen;
begin
  get_pen:=Canvas.Pen;
end;

procedure TPolygons.set_Trans(b: boolean);
begin
  Trans:=b;
  FixIt;
  Invalidate;
end;

procedure TPolygons.set_PolyCount(c: integer);
begin
  _PolyCount:=c;
  FixIt;
  Invalidate;
end;

procedure TPolygons.set_StartPolyCount(c: integer);
begin
  _StartPolyCount:=c;
  Fixit;
  Invalidate;
end;

procedure TPolygons.set_WidthChange(w: integer);
begin
  _WidthChange:=w;
  if _WidthHeightChange=True then _HeightChange:=w;
  FixIt;
  Invalidate;
end;

procedure TPolygons.set_HeightChange(h: integer);
begin
  _HeightChange:=h;
  if _WidthHeightChange=True then _WidthChange:=h;
  FixIt;
  Invalidate;
end;

procedure TPolygons.set_StartAngleChange(s: integer);
begin
  if s<360 then _StartAngleChange:=s;
  FixIt;
  Invalidate;
end;

procedure TPolygons.set_AutoPolyCount(b: boolean);
begin
  _AutoPolyCount:=b;
  FixIt;
  Invalidate;
end;

procedure TPolygons.set_WidthHeightChange(b:boolean);
begin
  _WidthHeightChange:=b;
  _WidthChange:=_HeightChange;
  FixIt;
  Invalidate;
end;

procedure TPolygons.FixIt;
begin
  if _AutoPolyCount=True then
  begin
    if _WidthChange=0 then _WidthChange:=1;
    if _HeightChange=0 then _HeightChange:=1;
  end;
  if (_StartPolyCount>_PolyCount) and (AutoPolyCount=false) then _StartPolyCount:=_PolyCount;
end;
end.
